expanded class INPUT_OUTPUT[K]

feature -- output

	put (graph: GRAPH[K, EDGES[K]]; stream: OUTPUT_STREAM) is
	local
		verts: ITERATOR[VERTEX[K]]
		edges: ITERATOR[EDGE[K]]
		label: STRING
	do
		stream.put_string (graph.output_header)
		stream.put_string (" {%N")
		from
			verts := graph.get_new_iterator
		until
			verts.is_off
		loop
			stream.put_string ("%T" + graph.output_vertex (verts.item))
			label ?= verts.item.item
			if label /= Void then
				-- quito las ""
				label.remove_head (1)
				label.remove_tail (1)
				stream.put_string(" [label = '" + label + "']")
			end
			stream.put_string (";")
			stream.put_new_line
			verts.next
		end
		from
			edges := graph.get_new_iterator_on_edges
		until
			edges.is_off
		loop
			stream.put_string ("%T" + graph.output_edge (edges.item.source, edges.item.target) + ";")
			stream.put_new_line
			edges.next
		end
		stream.put_string ("}%N")
	end

feature -- Input

	read (stream: INPUT_STREAM; graph: GRAPH[K, EDGES[K]]) is
	local
		vertices: DICTIONARY[VERTEX[K], K]
		source, target: VERTEX[K]
		words: ARRAY[STRING]
		edge: EDGE[K]
		w_edge: WEIGHTED_EDGE[INTEGER, K]
		label: K
	do
		create vertices.make
		-- busca el comienzo del grafo
		from
			create words.make (1, 0)
		until
			words.count > 0 and then (words@1).has_substring ("graph")
		loop
			read_line (stream, words)
		end
		-- se fija si es dirigido
		if equal(words@1, "graph") then
			-- es no dirigido
		else
			-- es dirigido
		end
		-- agrega nodos y aristas
		from
			read_line (stream, words)
		until
			stream.end_of_input or words.count <= 0
		loop
			if words.count > 1 and then not equal (words@2, "label") then
				-- es arista
				source := vertices.at (words@1)
				target := vertices.at (words@2)
				if words.count > 3 then
					-- con peso
					create w_edge.make (source, target, (words@4).to_integer)
					graph.put_edge (w_edge)
				else
					-- sin peso
					create edge.make (source, target)
					graph.put_edge (edge)
				end
			else
				-- es nodo
				if words.count > 1 then
					-- tiene label
					label ?= words@3
					graph.add (label)
				else
					-- no tiene label
					graph.add (Void)
				end
				vertices.put (graph.last_vertex_added, words @ 1)
			end
			read_line (stream, words)
		end
	end

feature {NONE} -- Internal

	node: STRING

	tick: CHARACTER is '"'

	separators: STRING is ";{}[]'=%T<->"

	read_line (stream: INPUT_STREAM; words: ARRAY[STRING]) is
	do
		words.clear
		stream.read_line
		split_in (stream.last_string, words)
	end

	split_in(s: STRING; words: ARRAY[STRING]) is
	require
		s /= Void
	local
		state, i: INTEGER
		-- state = 0: waiting next word.
		-- state = 1: inside a new word.
		c: CHARACTER
		string_buffer: STRING
	do
		string_buffer := ""
		if s.count > 0 then
			from
				i := 1
			until
				i > s.count
			loop
				c := s.item(i)
				if state = 0 then
					if not separators.has(c) then
						string_buffer.clear
						string_buffer.append_character(c)
						state := 1
					end
				else
					if not separators.has (c) then
						string_buffer.append_character(c)
					else
						string_buffer.left_adjust
						string_buffer.right_adjust
						if not string_buffer.is_empty then
							words.add_last(string_buffer.twin)
						end
						state := 0
					end
				end
				i := i + 1
			end
			if state = 1 then
				string_buffer.left_adjust
				string_buffer.right_adjust
				if not string_buffer.is_empty then
					words.add_last(string_buffer.twin)
				end
			end
		end
	end

end -- class GRAPH_INPUT_OUTPUT
