deferred class SIMPLE_GRAPH[E->EDGES[INTEGER]]
	-- VERTICES + EDGES

inherit
	GRAPH[INTEGER, E]
		undefine create_vertex, has_vertex, has, fast_has, vertex_of, fast_vertex_of
		end
	SIMPLE_VERTICES
		redefine simple_make
		end

feature {NONE} -- Creation

	simple_make (l, u: INTEGER) is
	do
		Precursor (l, u)
		make
	end

feature -- edge operations

	simple_add_edge(v1, v2: INTEGER) is
		-- agrega una arista entre `v1' y `v2'. Si ya existe, no hace nada.
	require
		simple_is_valid_edge (v1, v2)
	do
		add_edge (create_vertex (v1), create_vertex (v2))
	ensure
		simple_are_adjacents(v1, v2)
	end

	simple_remove_edge(v1, v2: INTEGER) is
		-- quita la arista entre `v1' y `v2'
	do
		remove_edge (create_vertex (v1), create_vertex (v2))
	ensure
		not simple_are_adjacents(v1, v2)
	end

	simple_are_adjacents(v1, v2: INTEGER): BOOLEAN is
		-- hay una arista entre `v1' y `v2'?
	do
		Result := are_adjacents (create_vertex (v1), create_vertex(v2))
	ensure
		Result implies simple_is_valid_edge (v1, v2)
	end

	simple_adjacents(v: INTEGER): ITERATOR[INTEGER] is
		-- { w : simple_are_adjacents (v, w) }
	require
		simple_has_vertex(v)
	do
		create {SIMPLE_ITERATOR} Result.make (adjacents (create_vertex (v)))
	end

	simple_valency (v: INTEGER): INTEGER is
		-- cantidad de adjacents a `v'
	require
		simple_has_vertex(v)
	do
		Result := edges.count (create_vertex (v))
		Result := edges.simple_count (v)
	end

feature -- Paths

	simple_exists_path(v1, v2: INTEGER): BOOLEAN is
		-- hay un camino desde `v1' a `v2'
	require
		simple_has_vertex(v1)
	do
		Result := exists_path (create_vertex (v1), create_vertex(v2))
	end

	simple_distance (v1, v2: INTEGER): INTEGER is
		-- longitud del camino minimo desde `v1' a `v2'
	require
		simple_exists_path (v1, v2)
	do
		Result := distance (create_vertex (v1), create_vertex(v2))
	end

	simple_eulerian_path (v1, v2: INTEGER): BOOLEAN is
		-- hay un recorrido euleriano de `v1' a `v2'
	require
		simple_has_vertex (v1)
		simple_has_vertex (v1)
	do
		Result := eulerian_path (create_vertex (v1), create_vertex(v2))
	ensure
		Result implies is_connected
		Result and v1 == v2 implies simple_valency (v1) mod 2 == 0
		Result and v1 /= v2 implies simple_valency (v1) mod 2 == 1 and simple_valency (v2) mod 2 == 1
	end

feature -- Validation

	simple_is_valid_edge (v1, v2: INTEGER): BOOLEAN is
		-- es (`v1', `v2') una arista valida? Esto permite establecer restricciones. 
		-- Por ejemplo, en un grafo bipartito no se pueden unir dos v\'ertices de 
		-- la misma parte. En un \'arbol, no se puede agregar una arista si genera 
		-- un ciclo.
	do
		Result := is_valid_edge (create_vertex (v1), create_vertex(v2))
	ensure
		Result implies simple_has_vertex (v1) and simple_has_vertex (v2)
	end

end -- class SIMPLE_GRAPH[E]
