deferred class VERTICES[K]
	-- una coleccion de vertices

inherit MODEL

feature {NONE} -- Creation

	make is
		-- inicializa la coleccion
	deferred
	ensure
		count = 0
	end

feature -- Creation

	from_collection (c: like Current) is
		-- crea a partir de `c'
	local
		ii: ITERATOR[VERTEX[K]]
	do
		make
		from
			ii := c.get_new_iterator
		until
			ii.is_off
		loop
			add_vertex (ii.item)
			ii.next
		end
	end

feature -- access

	count: INTEGER is
		-- cantidad de vertices
	deferred
	end

	is_empty: BOOLEAN is
		-- no hay vertices
	do
		Result := count = 0
	end

feature -- vertex operations

	add (k: K) is
		-- crea un vertex con el contenido `k' y lo agrega al grafo
	do
		add_vertex (create_vertex (k))
	ensure
		last_vertex_added.item = k
	end

	last_vertex_added: VERTEX[K] is
		-- ultimo vertice agregado a la coleccion
	deferred
	end

feature -- Looking and searching

	get_new_iterator: ITERATOR[VERTEX[K]] is
		-- itera sobre los vertices en la coleccion
	deferred
	end

	has_vertex (v: VERTEX[K]): BOOLEAN is
		-- esta `v' en el grafo?
	deferred
	end

	has(k: K): BOOLEAN is
		-- hay un vertices con contenido `k'?
		-- use `equal' comparison
	local
		ii: ITERATOR[VERTEX[K]]
	do
		from ii := get_new_iterator
		until ii.is_off or else ii.item.item.is_equal (k)
		loop
			ii.next
		end
		Result := not ii.is_off
	end

	fast_has(k: K): BOOLEAN is
		-- hay un vertices con contenido `k'?
		-- use `=' comparison
	local
		ii: ITERATOR[VERTEX[K]]
	do
		from ii := get_new_iterator
		until ii.is_off or else ii.item.item = k
		loop
			ii.next
		end
		Result := not ii.is_off
	end

	vertex_of (k: K): VERTEX[K] is
		-- vertices cuyo contenido es `k'
		-- use `equal' comparison
	local
		ii: ITERATOR[VERTEX[K]]
	do
		from ii := get_new_iterator
		until ii.is_off or else ii.item.item.is_equal (k)
		loop
			ii.next
		end
		if not ii.is_off then
			Result := ii.item
		end
	end

	fast_vertex_of (k: K): VERTEX[K] is
		-- vertices cuyo contenido es `k'
		-- use `=' comparison
	local
		ii: ITERATOR[VERTEX[K]]
	do
		from ii := get_new_iterator
		until ii.is_off or else ii.item.item = k
		loop
			ii.next
		end
		if not ii.is_off then
			Result := ii.item
		end
	end

feature {VERTEX} -- Internal

	add_vertex (v: VERTEX[K]) is
		-- agrega el vertex `v' al grafo.
	deferred
	ensure
		has_vertex (v)
		last_vertex_added = v
	end

feature {NONE} -- Internal

	create_vertex (k: K): VERTEX[K] is
		-- crea un vertice con contenido `k'
	deferred
	ensure
		Result.item = k
	end

end -- class VERTICES[K]
